@@ -1,5 +1,5 @@
 #!/usr/bin/python
-import re, json, copy
+import re, json, copy, sys
 from main import *
 
 ### Hex to bin converter and vice versa for objects
@@ -43,7 +43,7 @@ def deserialize(tx):
         pos[0] += 1
         if ord(tx[pos[0]-1]) < 253: return ord(tx[pos[0]-1])
         return read_as_int(pow(2,ord(tx[pos[0]-1]) - 252))
-        
+
     def read_bytes(bytez):
         pos[0] += bytez
         return tx[pos[0]-bytez:pos[0]]
@@ -101,7 +101,7 @@ SIGHASH_ANYONECANPAY = 80
 def signature_form(tx, i, script, hashcode = SIGHASH_ALL):
     i, hashcode = int(i), int(hashcode)
     if isinstance(tx,str):
-        return serialize(signature_form(deserialize(tx),i,script))
+        return serialize(signature_form(deserialize(tx),i,script,hashcode))
     newtx = copy.deepcopy(tx)
     for inp in newtx["ins"]: inp["script"] = ""
     newtx["ins"][i]["script"] = script
@@ -168,7 +168,7 @@ def mk_scripthash_script(addr):
 
 # Address representation to output script
 def address_to_script(addr):
-    if addr[0] == '3': return mk_scripthash_script(addr)
+    if addr[0] == '3' or addr[0] == '2': return mk_scripthash_script(addr)
     else: return mk_pubkey_script(addr)
 
 # Output script to address representation
@@ -178,13 +178,22 @@ def script_to_address(script,vbyte=0):
     if script[:3] == '\x76\xa9\x14' and script[-2:] == '\x88\xac' and len(script) == 25:
         return bin_to_b58check(script[3:-2],vbyte) # pubkey hash addresses
     else:
-        return bin_to_b58check(script[2:-1],5) # BIP0016 scripthash addresses
+        if vbyte == 111:
+            # Testnet
+            scripthash_byte = 192
+        else:
+            scripthash_byte = 5
+            
+        return bin_to_b58check(script[2:-1],scripthash_byte) # BIP0016 scripthash addresses
 
-def p2sh_scriptaddr(script):
-    if re.match('^[0-9a-fA-F]*$',script): script = script.decode('hex')
-    return hex_to_b58check(hash160(script),5)
+
+def p2sh_scriptaddr(script, magicbyte=5):
+    if re.match('^[0-9a-fA-F]*$', script):
+        script = script.decode('hex')
+    return hex_to_b58check(hash160(script), magicbyte)
 scriptaddr = p2sh_scriptaddr
 
+
 def deserialize_script(script):
     if re.match('^[0-9a-fA-F]*$',script):
         return json_changebase(deserialize_script(script.decode('hex')),lambda x:x.encode('hex'))
@@ -227,10 +236,14 @@ def serialize_script(script):
         return serialize_script(json_changebase(script,lambda x:x.decode('hex'))).encode('hex')
     return ''.join(map(serialize_script_unit,script))
 
-def mk_multisig_script(*args): # [pubs],k,n or pub1,pub2...pub[n],k,n
-    if len(args) == 3: pubs, k, n = args[0], int(args[1]), int(args[2])
-    else: pubs, k, n = list(args[:-2]), int(args[-2]), int(args[-1])
-    return serialize_script([k]+pubs+[n,174])
+
+def mk_multisig_script(*args):  # [pubs],k or pub1,pub2...pub[n],k
+    if isinstance(args[0], list):
+        pubs, k = args[0], int(args[1])
+    else:
+        pubs = filter(lambda x: len(str(x)) >= 32, args)
+        k = args[pubs]
+    return serialize_script([k]+pubs+[len(pubs), 174])
 
 ### Signing and verifying
 
@@ -240,7 +253,7 @@ def verify_tx_input(tx,i,script,sig,pub):
     if not re.match('^[0-9a-fA-F]*$',sig): sig = sig.encode('hex')
     hashcode = decode(sig[-2:],16)
     modtx = signature_form(tx,int(i),script,hashcode)
-    return ecdsa_tx_verify(modtx,sig,pub)
+    return ecdsa_tx_verify(modtx,sig,pub,hashcode)
 
 def sign(tx,i,priv):
     i = int(i)
@@ -260,11 +273,11 @@ def signall(tx,priv):
         tx = sign(tx,i,priv)
     return tx
 
-def multisign(tx,i,script,pk):
+def multisign(tx,i,script,pk,hashcode = SIGHASH_ALL):
     if re.match('^[0-9a-fA-F]*$',tx): tx = tx.decode('hex')
     if re.match('^[0-9a-fA-F]*$',script): script = script.decode('hex')
-    modtx = signature_form(tx,i,script)
-    return ecdsa_tx_sign(modtx,pk)
+    modtx = signature_form(tx,i,script,hashcode)
+    return ecdsa_tx_sign(modtx,pk,hashcode)
 
 def apply_multisignatures(*args): # tx,i,script,sigs OR tx,i,script,sig1,sig2...,sig[n]
     tx, i, script = args[0], int(args[1]), args[2]
@@ -296,10 +309,10 @@ def mktx(*args): # [in0, in1...],[out0, out1...] or in0, in1 ... out0 out1 ...
             txobj["ins"].append(i)
         else:
             if isinstance(i,dict) and "output" in i: i = i["output"]
-            txobj["ins"].append({ 
+            txobj["ins"].append({
                 "outpoint" : { "hash": i[:64], "index": int(i[65:]) },
                 "script": "",
-                "sequence": 4294967295 
+                "sequence": 4294967295
             })
     for o in outs:
         if isinstance(o,str): o = {
@@ -346,7 +359,7 @@ def mksend(*args):
         else: o2 = o
         outputs2.append(o2)
         osum += o2["value"]
-        
+
     if isum < osum+fee:
         raise Exception("Not enough money")
     elif isum > osum+fee+5430:
